import { isUndefined } from 'lodash';
import React, { useContext, useEffect, useState } from 'react';
import { useEventCallback } from '@react-devui/hooks';
export const DFormUpdateContext = React.createContext(null);
export function useForm(cb) {
    const [form, setForm] = useState(() => cb());
    const updateForm = useEventCallback((val) => {
        isUndefined(val) ? setForm((draft) => draft.clone()) : setForm(val);
    });
    return [form, updateForm];
}
export function useFormControl(formControl) {
    const updateForm = useContext(DFormUpdateContext);
    const control = formControl === null || formControl === void 0 ? void 0 : formControl.control;
    useEffect(() => {
        if (control) {
            const ob = control.asyncVerifyComplete$.subscribe({
                next: (val) => {
                    if (val.dirty) {
                        updateForm === null || updateForm === void 0 ? void 0 : updateForm();
                    }
                },
            });
            return () => {
                ob.unsubscribe();
            };
        }
    }, [control, updateForm]);
    return control
        ? [
            control.value,
            (val) => {
                if (control) {
                    control.markAsDirty(true);
                    control.setValue(val);
                    updateForm === null || updateForm === void 0 ? void 0 : updateForm();
                }
            },
        ]
        : undefined;
}
